(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2005-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 3.3+dev4 generated code *)

let print_native_int ppf = function | n_t -> Nativeint_print.print_t ppf n_t

and print_int32 ppf = function | i_t -> Int32_print.print_t ppf i_t

and print_int64 ppf = function | i_t -> Int64_print.print_t ppf i_t
;;

let rec print_t ppf = function
  | Typed_value.Scalar s ->
    Format.fprintf ppf "@[<1>%s@ " "Scalar"; print_scalar ppf s;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vector v ->
    Format.fprintf ppf "@[<1>%s@ " "Vector"; print_vector ppf v;
    Format.fprintf ppf "%,@]"
  | Typed_value.Matrix m ->
    Format.fprintf ppf "@[<1>%s@ " "Matrix"; print_matrix ppf m;
    Format.fprintf ppf "%,@]"

and print_scalar ppf = function
  | Typed_value.Sbool b ->
    Format.fprintf ppf "@[<1>%s@ " "Sbool"; Lib_print.print_bool ppf b;
    Format.fprintf ppf "%,@]"
  | Typed_value.Sbool_couple (b, b0) ->
    Format.fprintf ppf "@[<1>%s@ " "Sbool_couple";
    Format.fprintf ppf "%,@[<1>(@,"; Lib_print.print_bool ppf b;
    Format.fprintf ppf "%,,@ "; Lib_print.print_bool ppf b0;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Typed_value.Sbool_triple (b, b0, b1) ->
    Format.fprintf ppf "@[<1>%s@ " "Sbool_triple";
    Format.fprintf ppf "%,@[<1>(@,"; Lib_print.print_bool ppf b;
    Format.fprintf ppf "%,,@ "; Lib_print.print_bool ppf b0;
    Format.fprintf ppf "%,,@ "; Lib_print.print_bool ppf b1;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Typed_value.Sstring s ->
    Format.fprintf ppf "@[<1>%s@ " "Sstring";
    Lib_print.print_quoted_string ppf s; Format.fprintf ppf "%,@]"
  | Typed_value.Sint i ->
    Format.fprintf ppf "@[<1>%s@ " "Sint"; Lib_print.print_quoted_int ppf i;
    Format.fprintf ppf "%,@]"
  | Typed_value.Sint_couple (i, i0) ->
    Format.fprintf ppf "@[<1>%s@ " "Sint_couple";
    Format.fprintf ppf "%,@[<1>(@,"; Lib_print.print_quoted_int ppf i;
    Format.fprintf ppf "%,,@ "; Lib_print.print_quoted_int ppf i0;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Typed_value.Sint_triple (i, i0, i1) ->
    Format.fprintf ppf "@[<1>%s@ " "Sint_triple";
    Format.fprintf ppf "%,@[<1>(@,"; Lib_print.print_quoted_int ppf i;
    Format.fprintf ppf "%,,@ "; Lib_print.print_quoted_int ppf i0;
    Format.fprintf ppf "%,,@ "; Lib_print.print_quoted_int ppf i1;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Typed_value.Snative_int n ->
    Format.fprintf ppf "@[<1>%s@ " "Snative_int"; print_native_int ppf n;
    Format.fprintf ppf "%,@]"
  | Typed_value.Snative_int_couple (n, n0) ->
    Format.fprintf ppf "@[<1>%s@ " "Snative_int_couple";
    Format.fprintf ppf "%,@[<1>(@,"; print_native_int ppf n;
    Format.fprintf ppf "%,,@ "; print_native_int ppf n0;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Typed_value.Snative_int_triple (n, n0, n1) ->
    Format.fprintf ppf "@[<1>%s@ " "Snative_int_triple";
    Format.fprintf ppf "%,@[<1>(@,"; print_native_int ppf n;
    Format.fprintf ppf "%,,@ "; print_native_int ppf n0;
    Format.fprintf ppf "%,,@ "; print_native_int ppf n1;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Typed_value.Sint32 i ->
    Format.fprintf ppf "@[<1>%s@ " "Sint32"; print_int32 ppf i;
    Format.fprintf ppf "%,@]"
  | Typed_value.Sint32_couple (i, i0) ->
    Format.fprintf ppf "@[<1>%s@ " "Sint32_couple";
    Format.fprintf ppf "%,@[<1>(@,"; print_int32 ppf i;
    Format.fprintf ppf "%,,@ "; print_int32 ppf i0;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Typed_value.Sint32_triple (i, i0, i1) ->
    Format.fprintf ppf "@[<1>%s@ " "Sint32_triple";
    Format.fprintf ppf "%,@[<1>(@,"; print_int32 ppf i;
    Format.fprintf ppf "%,,@ "; print_int32 ppf i0;
    Format.fprintf ppf "%,,@ "; print_int32 ppf i1;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Typed_value.Sint64 i ->
    Format.fprintf ppf "@[<1>%s@ " "Sint64"; print_int64 ppf i;
    Format.fprintf ppf "%,@]"
  | Typed_value.Sint64_couple (i, i0) ->
    Format.fprintf ppf "@[<1>%s@ " "Sint64_couple";
    Format.fprintf ppf "%,@[<1>(@,"; print_int64 ppf i;
    Format.fprintf ppf "%,,@ "; print_int64 ppf i0;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Typed_value.Sint64_triple (i, i0, i1) ->
    Format.fprintf ppf "@[<1>%s@ " "Sint64_triple";
    Format.fprintf ppf "%,@[<1>(@,"; print_int64 ppf i;
    Format.fprintf ppf "%,,@ "; print_int64 ppf i0;
    Format.fprintf ppf "%,,@ "; print_int64 ppf i1;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Typed_value.Sfloat f ->
    Format.fprintf ppf "@[<1>%s@ " "Sfloat";
    Lib_print.print_quoted_float ppf f; Format.fprintf ppf "%,@]"
  | Typed_value.Sfloat_couple (f, f0) ->
    Format.fprintf ppf "@[<1>%s@ " "Sfloat_couple";
    Format.fprintf ppf "%,@[<1>(@,"; Lib_print.print_quoted_float ppf f;
    Format.fprintf ppf "%,,@ "; Lib_print.print_quoted_float ppf f0;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Typed_value.Sfloat_triple (f, f0, f1) ->
    Format.fprintf ppf "@[<1>%s@ " "Sfloat_triple";
    Format.fprintf ppf "%,@[<1>(@,"; Lib_print.print_quoted_float ppf f;
    Format.fprintf ppf "%,,@ "; Lib_print.print_quoted_float ppf f0;
    Format.fprintf ppf "%,,@ "; Lib_print.print_quoted_float ppf f1;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"

and print_vector ppf = function
  | Typed_value.Vbool b_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vbool";
    Lib_print.print_Array Lib_print.print_bool ppf b_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vbool_couple t_b_b_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vbool_couple";
    Lib_print.print_Array
     ((fun ppf (b, b0) ->
       Format.fprintf ppf "%,@[<1>(@,"; Lib_print.print_bool ppf b;
       Format.fprintf ppf "%,,@ "; Lib_print.print_bool ppf b0;
       Format.fprintf ppf "%,@,)@]")
     ) ppf t_b_b_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vbool_triple t_b_b_b_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vbool_triple";
    Lib_print.print_Array
     ((fun ppf (b, b0, b1) ->
       Format.fprintf ppf "%,@[<1>(@,"; Lib_print.print_bool ppf b;
       Format.fprintf ppf "%,,@ "; Lib_print.print_bool ppf b0;
       Format.fprintf ppf "%,,@ "; Lib_print.print_bool ppf b1;
       Format.fprintf ppf "%,@,)@]")
     ) ppf t_b_b_b_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vstring s_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vstring";
    Lib_print.print_Array Lib_print.print_quoted_string ppf s_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vint i_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vint";
    Lib_print.print_Array Lib_print.print_quoted_int ppf i_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vint_couple t_i_i_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vint_couple";
    Lib_print.print_Array
     ((fun ppf (i, i0) ->
       Format.fprintf ppf "%,@[<1>(@,"; Lib_print.print_quoted_int ppf i;
       Format.fprintf ppf "%,,@ "; Lib_print.print_quoted_int ppf i0;
       Format.fprintf ppf "%,@,)@]")
     ) ppf t_i_i_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vint_triple t_i_i_i_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vint_triple";
    Lib_print.print_Array
     ((fun ppf (i, i0, i1) ->
       Format.fprintf ppf "%,@[<1>(@,"; Lib_print.print_quoted_int ppf i;
       Format.fprintf ppf "%,,@ "; Lib_print.print_quoted_int ppf i0;
       Format.fprintf ppf "%,,@ "; Lib_print.print_quoted_int ppf i1;
       Format.fprintf ppf "%,@,)@]")
     ) ppf t_i_i_i_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vnative_int n_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vnative_int";
    Lib_print.print_Array print_native_int ppf n_a; Format.fprintf ppf "%,@]"
  | Typed_value.Vnative_int_couple t_n_n_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vnative_int_couple";
    Lib_print.print_Array
     ((fun ppf (n, n0) ->
       Format.fprintf ppf "%,@[<1>(@,"; print_native_int ppf n;
       Format.fprintf ppf "%,,@ "; print_native_int ppf n0;
       Format.fprintf ppf "%,@,)@]")
     ) ppf t_n_n_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vnative_int_triple t_n_n_n_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vnative_int_triple";
    Lib_print.print_Array
     ((fun ppf (n, n0, n1) ->
       Format.fprintf ppf "%,@[<1>(@,"; print_native_int ppf n;
       Format.fprintf ppf "%,,@ "; print_native_int ppf n0;
       Format.fprintf ppf "%,,@ "; print_native_int ppf n1;
       Format.fprintf ppf "%,@,)@]")
     ) ppf t_n_n_n_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vint32 i_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vint32";
    Lib_print.print_Array print_int32 ppf i_a; Format.fprintf ppf "%,@]"
  | Typed_value.Vint32_couple t_i_i_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vint32_couple";
    Lib_print.print_Array
     ((fun ppf (i, i0) ->
       Format.fprintf ppf "%,@[<1>(@,"; print_int32 ppf i;
       Format.fprintf ppf "%,,@ "; print_int32 ppf i0;
       Format.fprintf ppf "%,@,)@]")
     ) ppf t_i_i_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vint32_triple t_i_i_i_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vint32_triple";
    Lib_print.print_Array
     ((fun ppf (i, i0, i1) ->
       Format.fprintf ppf "%,@[<1>(@,"; print_int32 ppf i;
       Format.fprintf ppf "%,,@ "; print_int32 ppf i0;
       Format.fprintf ppf "%,,@ "; print_int32 ppf i1;
       Format.fprintf ppf "%,@,)@]")
     ) ppf t_i_i_i_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vint64 i_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vint64";
    Lib_print.print_Array print_int64 ppf i_a; Format.fprintf ppf "%,@]"
  | Typed_value.Vint64_couple t_i_i_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vint64_couple";
    Lib_print.print_Array
     ((fun ppf (i, i0) ->
       Format.fprintf ppf "%,@[<1>(@,"; print_int64 ppf i;
       Format.fprintf ppf "%,,@ "; print_int64 ppf i0;
       Format.fprintf ppf "%,@,)@]")
     ) ppf t_i_i_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vint64_triple t_i_i_i_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vint64_triple";
    Lib_print.print_Array
     ((fun ppf (i, i0, i1) ->
       Format.fprintf ppf "%,@[<1>(@,"; print_int64 ppf i;
       Format.fprintf ppf "%,,@ "; print_int64 ppf i0;
       Format.fprintf ppf "%,,@ "; print_int64 ppf i1;
       Format.fprintf ppf "%,@,)@]")
     ) ppf t_i_i_i_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vfloat f_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vfloat";
    Lib_print.print_Array Lib_print.print_quoted_float ppf f_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vfloat_couple t_f_f_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vfloat_couple";
    Lib_print.print_Array
     ((fun ppf (f, f0) ->
       Format.fprintf ppf "%,@[<1>(@,"; Lib_print.print_quoted_float ppf f;
       Format.fprintf ppf "%,,@ "; Lib_print.print_quoted_float ppf f0;
       Format.fprintf ppf "%,@,)@]")
     ) ppf t_f_f_a;
    Format.fprintf ppf "%,@]"
  | Typed_value.Vfloat_triple t_f_f_f_a ->
    Format.fprintf ppf "@[<1>%s@ " "Vfloat_triple";
    Lib_print.print_Array
     ((fun ppf (f, f0, f1) ->
       Format.fprintf ppf "%,@[<1>(@,"; Lib_print.print_quoted_float ppf f;
       Format.fprintf ppf "%,,@ "; Lib_print.print_quoted_float ppf f0;
       Format.fprintf ppf "%,,@ "; Lib_print.print_quoted_float ppf f1;
       Format.fprintf ppf "%,@,)@]")
     ) ppf t_f_f_f_a;
    Format.fprintf ppf "%,@]"

and print_matrix ppf = function
  | Typed_value.Mcfloat f_u_t ->
    Format.fprintf ppf "@[<1>%s@ " "Mcfloat";
    Umatrix_print.print_t Lib_print.print_quoted_float ppf f_u_t;
    Format.fprintf ppf "%,@]"
  | Typed_value.Mffloat f_u_t ->
    Format.fprintf ppf "@[<1>%s@ " "Mffloat";
    Umatrix_print.print_t Lib_print.print_quoted_float ppf f_u_t;
    Format.fprintf ppf "%,@]"
;;

(* End of OCamlGen, 3.3+dev4 generated code *)
