(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: black_box.ml,v 1.58 2014-02-19 16:18:53 fclement Exp $ *)

module Nat = Unatural;;
module Ddgraph = Udelayed_dgraph;;

module Structural_mapping = Umatrix_structural_mapping;;

module Config = Configuration;;
module W = Wrapper;;

let piob_ref = ref None;;

let pf = Pio.foreman_make_printing_facilities ();;

open Scalar_spec;;
open Components_spec;;

module Make =
  functor (Scal : SCALAR) ->
  functor (Comps : COMPONENTS) ->
struct
  module T = Translate.Make (Scal) (Comps);;
  module Stt = T.Stt;;
  
  type scalar = Scal.t;;
  type 'a components = 'a Comps.t;;
  
  type gradients = Stt.gradients;;
  type partitions = Stt.partitions;;
  type state = Stt.state;;
  type selected_cutting = Stt.selected_cutting;;
  
  let wrap tw =
    let cmd = Config.get_external_program_command () in
    Pio.foreman_wrap_task cmd piob_ref tw
  ;;
  
  (* Init. *)
  
  let initialize x = wrap (W.make_init_wrapper pf "init") x;;
  
  let init () =
    let dim = T.inj_init () in
    let (sz, ppm, maa, j) = initialize dim in
    T.proj_coarse_init sz ppm maa j
  ;;
  
  (* Grad. *)
  
  let compute_gradient x = wrap (W.make_grad_wrapper pf "grad") x;;
  
  let grad st =
    let (t_inj_fine, t_proj_fine) = T.make_fine st in
    let pm = t_inj_fine () in
    let gm = compute_gradient pm in
    t_proj_fine gm
  ;;
  
  (* Optim. *)
  
  let optimize x = wrap (W.make_optim_wrapper pf "optim") x;;
  
  let optim st_init pps_init =
    let (t_inj_coarse, t_proj_coarse) = T.make_coarse st_init pps_init in
    let (ppm, maa_init) = t_inj_coarse () in
    let (maa_opt, j_opt) = optimize (ppm, maa_init) in
    t_proj_coarse maa_opt j_opt
  ;;
  
  (* Final. *)
  
  let finalize x = wrap (W.make_final_wrapper pf "final") x;;
  
  let final dname st =
    let (t_inj_fine, _) = T.make_fine st in
    let pm = t_inj_fine () in
    finalize (dname, pm)
  ;;
end
;;
